==> hermes-5.1.0.21.rec2.tar.gz: hermes-5.1.0.21.rec2/app/models/hermes/concerns/form_validator_concern.rb <==
# frozen_string_literal: true

# app/hermes/models/concerns/common_field.rb
# app/controllers/concerns/test_concern.rb

module Hermes
    module Concerns
        module FormValidatorConcern
            extend ActiveSupport::Concern

            included do
                field :form_sne_validation_errors, type: Array, default: []
                field :form_front_validation_errors, type: Array, default: []

                
                before_save :validate_fields

                def validate_fields
                    path = form_validator_csv_path
                    rule_to_checks = CSV.read(path)
                    version = rule_to_checks[0][1]
                    front_errors = []
                    sne_errors = []
                    rule_to_checks.each_with_index do |line_to_check,i|
                        if(i>1)
                            unless check_rule_to_check_line_to_check(line_to_check)
                                if line_to_check[6] == "1"
                                    sne_errors.push(format_error(line_to_check, i, version))
                                else
                                    front_errors.push(format_error(line_to_check, i, version))
                                end
                            end
                        end
                    end
                   
                    self.form_front_validation_errors = front_errors
                    self.form_sne_validation_errors = sne_errors
                end


                def check_rule_to_check_line_to_check(line_to_check)
                    if line_to_check[1].blank?
                        return true
                    else
                        verifified_only_if_condition = line_to_check[2]
                        case verifified_only_if_condition
                        when "none", nil, ""
                            verify_rule_to_check(line_to_check)
                        else
                            validate_line_to_check_and_condition(verifified_only_if_condition,line_to_check)
                        end
                    end
                end

                def verify_rule_to_check(line_to_check)
                    rule_to_check = line_to_check[1]
                    field = line_to_check[0]
                    case rule_to_check
                    when "presence"
                         return validate_presence(field)
                    else
                        return validate_complex_rule_to_check(field, rule_to_check)
                    end
                end

                def validate_line_to_check_and_condition(verifified_only_if_condition,line_to_check)
                    begin
                        rule_to_check = line_to_check[1]
                        collected_validation_results = []
                        if(verifified_only_if_condition.split('.')[0] == "applicant")
                            all_applicants_raw.each do |applicant|
                                if(eval(verifified_only_if_condition))
                                    case rule_to_check
                                    when "presence"
                                        collected_validation_results.push(validate_presence(line_to_check[0]))
                                    else
                                        collected_validation_results.push(eval(rule_to_check))
                                    end
                                   
                                end
                            end
                        else
                            reformatted_verifified_only_if_condition = verifified_only_if_condition.split("housing_request.")[1]
                            if(eval(reformatted_verifified_only_if_condition))
                                reformatted_rule_to_check = rule_to_check.split("housing_request.")[1]
                                collected_validation_results.push( eval(reformatted_rule_to_check) )
                            end
                        end
                        
                        return  !collected_validation_results.any?{|val|  val == false}
                    rescue  => e
                        ap e
                        ap 'error with condition' 
                        ap line_to_check
                    end
                end
                def validate_presence(field)
                    array_values = get_values_from_object_path(field)|| []
                    array_values.any? { |val| !val.blank?}
                end

                def validate_complex_rule_to_check(field, rule_to_check)
                    begin
                        collected_validation_results = []
                        if(rule_to_check.split('.')[0] == "applicant")
                            all_applicants_raw.each do |applicant|
                                collected_validation_results.push(eval(rule_to_check))
                            end
                        else
                            reformatted_rule_to_check = rule_to_check.split("housing_request.")[1]
                            collected_validation_results.push( eval(reformatted_rule_to_check) )
                        end
                        return  !collected_validation_results.any?{|val|  val == false}
                    rescue  => e
                        ap e
                        ap 'error with rule_to_check' 
                        ap field
                    end
                end

                def get_values_from_object_path(field)
                    begin
                        array_path = field.split(".")
                        collected_values =[]
                        if(array_path[0] == "applicant")
                        
                            all_applicants_raw.each do |applicant|
                                    collected_values.push(eval(field))
                            end
                        else
                            collected_values.push(eval(field.split("housing_request.")[1]))
                        end
                        collected_values
                    rescue => e
                        ap e
                        ap 'error with field recuperation' 
                        ap field
                    end

                end

                def format_alin_field(label)
                    label.sub! "[year-1]" , (Date.current.year - 1 ).to_s
                    label.sub! "[year-2]" , (Date.current.year - 2 ).to_s
                    label
                end
                def format_error(line_to_check, i, version)
                    {
                        "category" => line_to_check[3],
                        "sub_category" => line_to_check[4],      
                        "alin_field" => format_alin_field(line_to_check[5]),       
                        "csv_rule_to_check_line_to_check" => (i+1).to_s,                
                        "hermes_field" => line_to_check[0],     
                        "rule_to_check" => line_to_check[1],                
                        "verification_condition" =>line_to_check[2] ,     
                        "csv_rule_to_checks_version" => version,
                        "is_sne" => line_to_check[6]          
                    }
                end
            end
        end
    end
end

==> hermes-5.1.0.24.rec2.tar.gz: hermes-5.1.0.24.rec2/app/models/hermes/concerns/form_validator_concern.rb <==
# frozen_string_literal: true

# app/hermes/models/concerns/common_field.rb
# app/controllers/concerns/test_concern.rb

module Hermes
    module Concerns
        module FormValidatorConcern
            extend ActiveSupport::Concern

            included do
                field :form_sne_validation_errors, type: Array, default: []
                field :form_front_validation_errors, type: Array, default: []

                
                before_save :validate_fields

                def validate_fields
                    path = form_validator_csv_path
                    rule_to_checks = CSV.read(path)
                    version = rule_to_checks[0][1]
                    front_errors = []
                    sne_errors = []
                    rule_to_checks.each_with_index do |line_to_check,i|
                        if(i>1)
                            unless check_rule_to_check_line_to_check(line_to_check)
                                if line_to_check[6] == "1"
                                    sne_errors.push(format_error(line_to_check, i, version))
                                else
                                    front_errors.push(format_error(line_to_check, i, version))
                                end
                            end
                        end
                    end
                   
                    self.form_front_validation_errors = front_errors
                    self.form_sne_validation_errors = sne_errors
                end


                def check_rule_to_check_line_to_check(line_to_check)
                    if line_to_check[1].blank?
                        return true
                    else
                        verifified_only_if_condition = line_to_check[2]
                        case verifified_only_if_condition
                        when "none", nil, ""
                            verify_rule_to_check(line_to_check)
                        else
                            validate_line_to_check_and_condition(verifified_only_if_condition,line_to_check)
                        end
                    end
                end

                def verify_rule_to_check(line_to_check)
                    rule_to_check = line_to_check[1]
                    field = line_to_check[0]
                    case rule_to_check
                    when "presence"
                         return validate_presence(field)
                    else
                        return validate_complex_rule_to_check(field, rule_to_check)
                    end
                end

                def validate_line_to_check_and_condition(verifified_only_if_condition,line_to_check)
                    begin
                        rule_to_check = line_to_check[1]
                        collected_validation_results = []
                        if(verifified_only_if_condition.split('.')[0] == "applicant")
                            all_applicants_raw.each do |applicant|
                                if(eval(verifified_only_if_condition))
                                    case rule_to_check
                                    when "presence"
                                        collected_validation_results.push(validate_presence(line_to_check[0]))
                                    else
                                        collected_validation_results.push(eval(rule_to_check))
                                    end
                                   
                                end
                            end
                        else
                            reformatted_verifified_only_if_condition = verifified_only_if_condition.split("housing_request.")[1]
                            if(eval(reformatted_verifified_only_if_condition))
                                reformatted_rule_to_check = rule_to_check.split("housing_request.")[1]
                                collected_validation_results.push( eval(reformatted_rule_to_check) )
                            end
                        end
                        
                        return  !collected_validation_results.any?{|val|  val == false}
                    rescue  => e
                        ap e
                        ap 'error with condition' 
                        ap line_to_check
                    end
                end
                def validate_presence(field)
                    array_values = get_values_from_object_path(field)|| []
                    array_values.any? { |val| val.is_a?(Boolean) ? !val.nil? : !val.blank?}
                end

                def validate_complex_rule_to_check(field, rule_to_check)
                    begin
                        collected_validation_results = []
                        if(rule_to_check.split('.')[0] == "applicant")
                            all_applicants_raw.each do |applicant|
                                collected_validation_results.push(eval(rule_to_check))
                            end
                        else
                            reformatted_rule_to_check = rule_to_check.split("housing_request.")[1]
                            collected_validation_results.push( eval(reformatted_rule_to_check) )
                        end
                        return  !collected_validation_results.any?{|val|  val == false}
                    rescue  => e
                        ap e
                        ap 'error with rule_to_check' 
                        ap field
                    end
                end

                def get_values_from_object_path(field)
                    begin
                        array_path = field.split(".")
                        collected_values =[]
                        if(array_path[0] == "applicant")
                        
                            all_applicants_raw.each do |applicant|
                                    collected_values.push(eval(field))
                            end
                        else
                            collected_values.push(eval(field.split("housing_request.")[1]))
                        end
                        collected_values
                    rescue => e
                        ap e
                        ap 'error with field recuperation' 
                        ap field
                    end

                end

                def format_alin_field(label)
                    label.sub! "[year-1]" , (Date.current.year - 1 ).to_s
                    label.sub! "[year-2]" , (Date.current.year - 2 ).to_s
                    label
                end
                def format_error(line_to_check, i, version)
                    {
                        "category" => line_to_check[3],
                        "sub_category" => line_to_check[4],      
                        "alin_field" => format_alin_field(line_to_check[5]),       
                        "csv_rule_to_check_line_to_check" => (i+1).to_s,                
                        "hermes_field" => line_to_check[0],     
                        "rule_to_check" => line_to_check[1],                
                        "verification_condition" =>line_to_check[2] ,     
                        "csv_rule_to_checks_version" => version,
                        "is_sne" => line_to_check[6]          
                    }
                end
            end
        end
    end
end
